
package dataspork.stat;

import dataspork.util.skModel;
import dataspork.util.KVectors;
import dataspork.util.DataListener;
import javax.swing.event.EventListenerList;
import java.io.*;
import java.text.DecimalFormat;

/** Computes the structure function ,S(k), from configuration data */

public class skData implements DataListener {

   protected int dims[];
   protected double config[];
   protected double boxsize[];

   protected double kgrid[];
   protected double sk[];
   protected double skmax;

   protected skModel skmodel;


   protected EventListenerList listenerList;

   public skData(int dims[],double config[],double boxsize[]){

      this.dims = dims;
      this.config = config;
      this.boxsize = boxsize;

      listenerList =new EventListenerList();
      skmodel = new skModel(100,0,dims[2]-1);

      skmodel.addListener(this);

      calcSk();
   }
   public void calcSk(){

       double kv[];
       
       int start = skmodel.getCutoff();
       int nshell = skmodel.getKShell();
       KVectors k = new KVectors(boxsize,nshell);
       kv = k.getKVectors();
       kgrid = k.getKMagnitude();
       sk = new double[kgrid.length];

       int d;
       int dim = dims[0];
       int npart = dims[1];
       int nconfig = dims[2];
       double r,csum,ssum;

     //  System.out.println("calc Here dim:"+dim+" npart:"+npart+" nconfig: "+nconfig+"\n");
     //  System.out.println("calc Here k:"+kgrid.length+"\n");
       for (int nk =0; nk < kgrid.length; nk++){
          sk[nk] = 0.0;
       }
      
       for (int nc=start; nc< nconfig; nc++){
          for (int nk =0; nk < kgrid.length; nk++){
             csum = 0.0; ssum=0.0; 
             for (int np=0; np< npart; np++){ 
                r = 0.0;
                for (d=0;d<dim;d++){
                   r +=  kv[nk*dim + d] * config[nc*npart*dim + np*dim + d];
                }
                csum += Math.cos(r);
                ssum += Math.sin(r);
             }
             sk[nk] += csum*csum + ssum*ssum;
          }
       }
       double fac = 1.0/(npart*(nconfig-start));
       skmax = 0.0;
       for (int nk=0; nk<kgrid.length;nk++){
          sk[nk] = sk[nk]*fac;
          if (sk[nk]>skmax) {skmax = sk[nk];}
      //    System.out.println("nk:"+nk+"k: "+kgrid[nk]+" S(k):"+sk[nk]+"\n");
       }
    

       
   }

   public double[] getSk(){
      return sk;
   }

   public double[] getKGrid(){
      return kgrid;
   }

   public skModel getSkModel(){
      return skmodel; 
   }

   public double skMax(){
      return skmax;
   }
  
   public void addListener(DataListener e){
      listenerList.add(DataListener.class,e);
   }

   public void fireListeners(Object o){
      Object[] listeners = listenerList.getListenerList();
      for (int i=listeners.length-2;i>=0; i-=2){
         if (o!=listeners[i+1]){
            ((DataListener)listeners[i+1]).dataChanged(this);
         }
      }
   }

   public void dataChanged(Object o){
      calcSk();
      fireListeners(o);
   }


   public void write(Writer w) throws IOException {
      DecimalFormat rf = new DecimalFormat("0.000E00");
      DecimalFormat gf = new DecimalFormat("0.0000E00");
      for (int i=0;i<kgrid.length;i++){
      String s =  new String(rf.format(kgrid[i]) +"   "+gf.format(sk[i])+" \n");
        w.write(s);
      }

      
   }


}
